MetaSlotOwner

Struct MetaSlotOwner 

pub struct MetaSlotOwner {
    pub metadata_perm: MetadataPerms,
    pub ref_count_perm: PermissionU64,
    pub in_list_perm: PermissionU64,
    pub slot_vaddr: Vaddr,
    pub usage: PageUsage,
    pub paths_in_pt: Set<TreePath<NR_ENTRIES>>,
}
Permissions that remain under the authority of MetaRegionOwners.

ref_count and in_list exist for the complete lifetime of the corresponding MetaSlot (i.e., 'static).

Fields

metadata_perm: MetadataPermsref_count_perm: PermissionU64in_list_perm: PermissionU64slot_vaddr: Vaddrusage: PageUsagepaths_in_pt: Set<TreePath<NR_ENTRIES>>

The set of tree paths at which this slot is referenced. For PT-node slots this is a singleton. For data-frame slots this tracks every location the frame is currently mapped — allowing a single frame to be mapped at multiple addresses.

Implementations

Source

impl MetaSlotOwner

pub open spec fn same_permissions(self, other: Self) -> bool

{
    &&& self.metadata_perm == other.metadata_perm
    &&& self.ref_count_perm == other.ref_count_perm
    &&& self.in_list_perm == other.in_list_perm

}
pub open spec fn ref_count(self) -> u64

{ self.ref_count_perm.value() }
pub open spec fn storage_perm(self) -> PointsTo<MetaSlotStorage>

{ self.metadata_perm.storage_perm }
pub open spec fn vtable_ptr_perm(self) -> PointsTo<usize>

{ self.metadata_perm.vtable_ptr_perm }
pub proof fn tracked_borrow_mut_metadata_perms(tracked &mut self) -> tracked res : &mut MetadataPerms

ensures
*res == old(self).metadata_perm,
*final(self)
    == (Self {
        metadata_perm: *final(res),
        ..*old(self)
    }),